@@ -1,4 +1,4 @@
-{-# Language OverloadedStrings #-}
+{-# Language OverloadedStrings, TemplateHaskell #-}
 {-|
 Module      : Client.Commands.Channel
 Description : Channel management command implementations
@@ -9,30 +9,31 @@ Maintainer  : emertens@gmail.com
 
 module Client.Commands.Channel (channelCommands) where
 
-import           Client.Commands.Arguments.Spec
-import           Client.Commands.TabCompletion
-import           Client.Commands.Types
-import           Client.Commands.WordCompletion
-import           Client.State
-import           Client.State.Channel
-import           Client.State.Focus
-import           Client.State.Network
-import           Client.UserHost
-import           Control.Applicative
-import           Control.Lens
-import           Control.Monad
-import           Data.Foldable (traverse_)
-import           Data.List.Split (chunksOf)
-import           Data.Maybe (fromMaybe)
-import           Data.Text (Text)
-import qualified Client.State.EditBox as Edit
-import qualified Data.HashMap.Strict as HashMap
-import qualified Data.Text as Text
-import           Irc.Commands
-import           Irc.Modes
-import           Irc.UserInfo
-import           Irc.Identifier
-import           LensUtils (setStrict)
+import Client.Commands.Arguments.Spec
+import Client.Commands.Docs (chanopDocs, cmdDoc)
+import Client.Commands.TabCompletion (activeNicks, noChannelTab, simpleChannelTab, simpleTabCompletion)
+import Client.Commands.Types
+import Client.Commands.WordCompletion (plainWordCompleteMode)
+import Client.State
+import Client.State.Channel (chanLists, chanModes, chanTopic, chanUsers)
+import Client.State.EditBox qualified as Edit
+import Client.State.Focus
+import Client.State.Network
+import Client.UserHost ( UserAndHost(UserAndHost) )
+import Control.Applicative (liftA2)
+import Control.Lens
+import Control.Monad (unless)
+import Data.Foldable (traverse_)
+import Data.HashMap.Strict qualified as HashMap
+import Data.List.Split (chunksOf)
+import Data.Maybe (fromMaybe, maybeToList)
+import Data.Text (Text)
+import Data.Text qualified as Text
+import Irc.Commands (ircInvite, ircKick, ircMode, ircPrivmsg, ircRemove)
+import Irc.Identifier (Identifier, mkId, idText)
+import Irc.Modes
+import Irc.Message (isNickChar)
+import Irc.UserInfo (UserInfo(UserInfo), renderUserInfo)
 
 channelCommands :: CommandSection
 channelCommands = CommandSection "IRC channel management"
@@ -40,78 +41,74 @@ channelCommands = CommandSection "IRC channel management"
   [ Command
       (pure "mode")
       (fromMaybe [] <$> optionalArg (extensionArg "[modes]" modeParamArgs))
-      "Sets IRC modes.\n\
-      \\n\
-      \Examples:\n\
-      \Setting a ban:           /mode +b *!*@hostname\n\
-      \Removing a quiet:        /mode -q *!*@hostname\n\
-      \Voicing two users:       /mode +vv user1 user2\n\
-      \Demoting an op to voice: /mode +v-o user1 user1\n\
-      \\n\
-      \When executed in a network window, mode changes are applied to your user.\n\
-      \When executed in a channel window, mode changes are applied to the channel.\n\
-      \\n\
-      \This command has parameter sensitive tab-completion.\n\
-      \\n\
-      \See also: /masks /channelinfo\n"
-    $ NetworkCommand cmdMode tabMode
+      $(chanopDocs `cmdDoc` "mode")
+    $ MaybeChatCommand cmdMode tabMode
 
   , Command
       (pure "masks")
       (simpleToken "mode")
-      "Show mask lists for current channel.\n\
-      \\n\
-      \Common \^Bmode\^B values:\n\
-      \\^Bb\^B: bans\n\
-      \\^Bq\^B: quiets\n\
-      \\^BI\^B: invite exemptions (op view only)\n\
-      \\^Be\^B: ban exemption (op view only)s\n\
-      \\n\
-      \To populate the mask lists for the first time use: /mode \^Bmode\^B\n\
-      \\n\
-      \See also: /mode\n"
+      $(chanopDocs `cmdDoc` "masks")
     $ ChannelCommand cmdMasks noChannelTab
 
   , Command
       (pure "invite")
       (simpleToken "nick")
-      "Invite a user to the current channel.\n"
+      $(chanopDocs `cmdDoc` "invite")
     $ ChannelCommand cmdInvite simpleChannelTab
 
   , Command
       (pure "topic")
       (remainingArg "message")
-      "Set the topic on the current channel.\n\
-      \\n\
-      \Tab-completion with no \^Bmessage\^B specified will load the current topic for editing.\n"
+      $(chanopDocs `cmdDoc` "topic")
     $ ChannelCommand cmdTopic tabTopic
 
   , Command
       (pure "kick")
       (liftA2 (,) (simpleToken "nick") (remainingArg "reason"))
-      "Kick a user from the current channel.\n\
-      \\n\
-      \See also: /kickban /remove\n"
+      $(chanopDocs `cmdDoc` "kick")
     $ ChannelCommand cmdKick simpleChannelTab
 
   , Command
       (pure "kickban")
       (liftA2 (,) (simpleToken "nick") (remainingArg "reason"))
-      "Ban and kick a user from the current channel.\n\
-      \\n\
-      \Users are banned by hostname match.\n\
-      \See also: /kick /remove\n"
+      $(chanopDocs `cmdDoc` "kickban")
     $ ChannelCommand cmdKickBan simpleChannelTab
 
+  , Command
+      (pure "quiet")
+      (simpleToken "nick|mask")
+      $(chanopDocs `cmdDoc` "quiet")
+    $ ChannelCommand cmdQuiet simpleChannelTab
+
   , Command
       (pure "remove")
       (liftA2 (,) (simpleToken "nick") (remainingArg "reason"))
-      "Remove a user from the current channel.\n\
-      \\n\
-      \Remove works like /kick except it results in a PART.\n\
-      \See also: /kick /kickban\n"
+      $(chanopDocs `cmdDoc` "remove")
     $ ChannelCommand cmdRemove simpleChannelTab
 
+  , Command
+      (pure "op")
+      (optionalArg $ simpleToken "[nick]")
+      $(chanopDocs `cmdDoc` "op")
+    $ ChannelCommand (cmdStatus "OP" "+o") simpleChannelTab
+
+  , Command
+      (pure "deop")
+      (optionalArg $ simpleToken "[nick]")
+      $(chanopDocs `cmdDoc` "deop")
+    $ ChannelCommand (cmdStatus "DEOP" "-o") simpleChannelTab
+
+  , Command
+      (pure "voice")
+      (optionalArg $ simpleToken "[nick]")
+      $(chanopDocs `cmdDoc` "voice")
+    $ ChannelCommand (cmdStatus "VOICE" "+v") simpleChannelTab
+
+  , Command
+      (pure "devoice")
+      (optionalArg $ simpleToken "[nick]")
+      $(chanopDocs `cmdDoc` "devoice")
+    $ ChannelCommand (cmdStatus "DEVOICE" "-v") simpleChannelTab
   ]
 
 cmdRemove :: ChannelCommand (String, String)
@@ -142,6 +139,16 @@ cmdKickBan channelId cs st (who,reason) =
      cs' <- sendModeration channelId cmds cs
      commandSuccessUpdateCS cs' st
 
+cmdQuiet :: ChannelCommand String
+cmdQuiet channelId cs st who
+  | elem 'q' $ view (csModeTypes . modesLists) cs = do
+    let
+      whoTxt = Text.pack who
+      mask = if Text.all isNickChar whoTxt then renderUserInfo (computeBanUserInfo (mkId whoTxt) cs) else whoTxt
+    cs' <- sendModeration channelId [ircMode channelId ["q", mask]] cs
+    commandSuccessUpdateCS cs' st
+  | otherwise = commandFailureMsg "no list mode q on network" st
+
 cmdInvite :: ChannelCommand String
 cmdInvite channelId cs st nick =
   do let freeTarget = has (csChannels . ix channelId . chanModes . ix 'g') cs
@@ -151,12 +158,6 @@ cmdInvite channelId cs st nick =
               else sendModeration channelId [cmd] cs
      commandSuccessUpdateCS cs' st
 
-commandSuccessUpdateCS :: NetworkState -> ClientState -> IO CommandResult
-commandSuccessUpdateCS cs st =
-  do let network = view csNetwork cs
-     commandSuccess
-       $ setStrict (clientConnection network) cs st
-
 cmdMasks :: ChannelCommand String
 cmdMasks channel cs st rest =
   case rest of
@@ -167,13 +168,27 @@ cmdMasks channel cs st rest =
            unless (connecting || listLoaded)
              (sendMsg cs (ircMode channel [Text.singleton mode]))
 
-           commandSuccess (changeSubfocus (FocusMasks mode) st)
+           commandSuccess (changeSubfocus (FocusMasks (view csNetwork cs) channel mode) st)
 
     _ -> commandFailureMsg "unknown mask mode" st
 
+cmdStatus :: Text -> Text -> ChannelCommand (Maybe String)
+cmdStatus servCmd modeChg chan cs st target
+  | useChanServ chan cs = do
+    let command = [servCmd, idText chan] ++ maybeToList targetText
+    sendMsg cs $ ircPrivmsg "ChanServ" $ Text.unwords command
+    commandSuccessUpdateCS cs st
+  | otherwise = do
+    let target' = fromMaybe (idText $ view csNick cs) targetText
+    sendMsg cs $ ircMode chan [modeChg, target']
+    commandSuccessUpdateCS cs st
+  where
+    targetText = Text.pack <$> target
+
 computeBanUserInfo :: Identifier -> NetworkState    -> UserInfo
 computeBanUserInfo who cs =
   case view (csUser who) cs of
+    -- TODO: Maybe have multiple ways of computing this.
     Nothing                     -> UserInfo who "*" "*"
     Just (UserAndHost _ host _) -> UserInfo "*" "*" host
 
@@ -194,22 +209,22 @@ tabTopic _ channelId cs st rest
 
   | otherwise = commandFailure st
 
-cmdMode :: NetworkCommand [String]
-cmdMode cs st xs = modeCommand (Text.pack <$> xs) cs st
+cmdMode :: MaybeChatCommand [String]
+cmdMode chan cs st xs = modeCommand chan (Text.pack <$> xs) cs st
 
 modeCommand ::
-  [Text] {- mode parameters -} ->
-  NetworkState                 ->
-  ClientState                  ->
+  Maybe Identifier {- channel -} ->
+  [Text] {- mode parameters -}   ->
+  NetworkState                   ->
+  ClientState                    ->
   IO CommandResult
-modeCommand modes cs st =
-  case view clientFocus st of
-
-    NetworkFocus _ ->
+modeCommand maybeChan modes cs st =
+  case maybeChan of
+    Nothing ->
       do sendMsg cs (ircMode (view csNick cs) modes)
          commandSuccess st
 
-    ChannelFocus _ chan ->
+    Just chan ->
       case modes of
         [] -> success False [[]]
         flags:params ->
@@ -234,13 +249,10 @@ modeCommand modes cs st =
                       else cs <$ traverse_ (sendMsg cs) cmds
              commandSuccessUpdateCS cs' st
 
-    _ -> commandFailure st
-
-tabMode :: Bool -> NetworkCommand String
-tabMode isReversed cs st rest =
-  case view clientFocus st of
-
-    ChannelFocus _ channel
+tabMode :: Bool -> MaybeChatCommand String
+tabMode isReversed maybeChan cs st rest =
+  case maybeChan of
+    Just channel
       | flags:params     <- Text.words (Text.pack rest)
       , Just parsedModes <- splitModes (view csModeTypes cs) flags params
       , let parsedModesWithParams =
@@ -254,9 +266,9 @@ tabMode isReversed cs st rest =
   where
     paramIndex = length $ words $ uncurry take $ clientLine st
 
-modeParamArgs :: ClientState -> String -> Maybe (Args ClientState [String])
-modeParamArgs st str =
-  case view clientFocus st of
+modeParamArgs :: ArgsContext -> String -> Maybe (Args ArgsContext [String])
+modeParamArgs ArgsContext{argsContextSt=st, argsContextFocus=focus} str =
+  case focus of
     Unfocused      -> Nothing
     NetworkFocus _ -> Just (pure [str])
     ChannelFocus net _ ->
