@@ -1,4 +1,6 @@
-{-# Language OverloadedStrings #-}
+{-# Language OverloadedStrings, TemplateHaskell #-}
+{-# LANGUAGE BlockArguments #-}
+{-# LANGUAGE ViewPatterns #-}
 {-|
 Module      : Client.Commands.Queries
 Description : Query command implementations
@@ -9,104 +11,118 @@ Maintainer  : emertens@gmail.com
 
 module Client.Commands.Queries (queryCommands) where
 
-import           Client.Commands.Arguments.Spec
-import           Client.Commands.TabCompletion
-import           Client.Commands.Types
-import           Client.State.Network (sendMsg)
-import qualified Data.Text as Text
-import           Irc.Commands
+import Client.Commands.Arguments.Spec (optionalArg, remainingArg, simpleToken, tokenArg)
+import Client.Commands.Docs (queriesDocs, cmdDoc)
+import Client.Commands.TabCompletion (noNetworkTab, simpleNetworkTab)
+import Client.Commands.Types (commandSuccess, commandSuccessUpdateCS, Command(Command), CommandImpl(NetworkCommand), CommandSection(CommandSection), NetworkCommand)
+import Client.State (changeSubfocus)
+import Client.State.Focus (Subfocus(FocusChanList, FocusWho))
+import Client.State.Network (sendMsg, csChannelList, clsElist, csPingStatus, _PingConnecting, csWhoReply, csNetwork)
+import Client.WhoReply (newWhoReply)
+import Control.Applicative (liftA2)
+import Control.Lens (has, set, view)
+import Control.Monad (unless)
+import Data.Maybe (fromMaybe, maybeToList)
+import Data.Text qualified as Text
+import Irc.Commands
 
 queryCommands :: CommandSection
 queryCommands = CommandSection "Queries"
 
   [ Command
       (pure "who")
-      (remainingArg "arguments")
-      "Send WHO query to server with given arguments.\n"
+      (optionalArg (liftA2 (,) (simpleToken "[channel|nick|mask]") (optionalArg (simpleToken "[options]"))))
+      $(queriesDocs `cmdDoc` "who")
     $ NetworkCommand cmdWho simpleNetworkTab
 
   , Command
       (pure "whois")
       (remainingArg "arguments")
-      "Send WHOIS query to server with given arguments.\n"
+      $(queriesDocs `cmdDoc` "whois")
     $ NetworkCommand cmdWhois simpleNetworkTab
 
   , Command
       (pure "whowas")
       (remainingArg "arguments")
-      "Send WHOWAS query to server with given arguments.\n"
+      $(queriesDocs `cmdDoc` "whowas")
     $ NetworkCommand cmdWhowas simpleNetworkTab
 
   , Command
       (pure "ison")
       (remainingArg "arguments")
-      "Send ISON query to server with given arguments.\n"
+      $(queriesDocs `cmdDoc` "ison")
     $ NetworkCommand cmdIson   simpleNetworkTab
 
   , Command
       (pure "userhost")
       (remainingArg "arguments")
-      "Send USERHOST query to server with given arguments.\n"
+      $(queriesDocs `cmdDoc` "userhost")
     $ NetworkCommand cmdUserhost simpleNetworkTab
 
   , Command
       (pure "time")
       (optionalArg (simpleToken "[servername]"))
-      "Send TIME query to server with given arguments.\n"
+      $(queriesDocs `cmdDoc` "time")
     $ NetworkCommand cmdTime simpleNetworkTab
 
   , Command
       (pure "stats")
       (remainingArg "arguments")
-      "Send STATS query to server with given arguments.\n"
+      $(queriesDocs `cmdDoc` "stats")
     $ NetworkCommand cmdStats simpleNetworkTab
 
   , Command
       (pure "lusers")
       (optionalArg (simpleToken "[servername]"))
-      "Send LUSERS query to a given server.\n"
+      $(queriesDocs `cmdDoc` "lusers")
     $ NetworkCommand cmdLusers simpleNetworkTab
 
   , Command
       (pure "users")
       (optionalArg (simpleToken "[servername]"))
-      "Send USERS query to a given server.\n"
+      $(queriesDocs `cmdDoc` "users")
     $ NetworkCommand cmdUsers simpleNetworkTab
 
   , Command
-      (pure "motd") (optionalArg (simpleToken "[servername]"))
-      "Send MOTD query to server.\n"
+      (pure "motd")
+      (optionalArg (simpleToken "[servername]"))
+      $(queriesDocs `cmdDoc` "motd")
     $ NetworkCommand cmdMotd simpleNetworkTab
 
   , Command
-      (pure "admin") (optionalArg (simpleToken "[servername]"))
-      "Send ADMIN query to server.\n"
+      (pure "admin")
+      (optionalArg (simpleToken "[servername]"))
+      $(queriesDocs `cmdDoc` "admin")
     $ NetworkCommand cmdAdmin simpleNetworkTab
 
   , Command
-      (pure "rules") (optionalArg (simpleToken "[servername]"))
-      "Send RULES query to server.\n"
+      (pure "rules")
+      (optionalArg (simpleToken "[servername]"))
+      $(queriesDocs `cmdDoc` "rules")
     $ NetworkCommand cmdRules simpleNetworkTab
 
   , Command
-      (pure "info") (pure ())
-      "Send INFO query to server.\n"
+      (pure "info")
+      (pure ())
+      $(queriesDocs `cmdDoc` "info")
     $ NetworkCommand cmdInfo noNetworkTab
 
   , Command
-      (pure "list") (remainingArg "arguments")
-      "Send LIST query to server.\n"
+      (pure "list")
+      (optionalArg (liftA2 (,) (tokenArg "[clientopts]" (const lsaParse)) (optionalArg (simpleToken "[elist]"))))
+      $(queriesDocs `cmdDoc` "list")
     $ NetworkCommand cmdList simpleNetworkTab
 
   , Command
       (pure "links")
       (remainingArg "arguments")
-      "Send LINKS query to server with given arguments.\n"
+      $(queriesDocs `cmdDoc` "links")
     $ NetworkCommand cmdLinks simpleNetworkTab
 
   , Command
-      (pure "version") (optionalArg (simpleToken "[servername]"))
-      "Send VERSION query to server.\n"
+      (pure "version")
+      (optionalArg (simpleToken "[servername]"))
+      $(queriesDocs `cmdDoc` "version")
     $ NetworkCommand cmdVersion simpleNetworkTab
 
   ]
@@ -123,10 +139,44 @@ cmdVersion cs st mbservername =
                                 Nothing -> ""
      commandSuccess st
 
-cmdList :: NetworkCommand String
+cmdList :: NetworkCommand (Maybe (ListArgs, Maybe String))
 cmdList cs st rest =
-  do sendMsg cs (ircList (Text.pack <$> words rest))
-     commandSuccess st
+    do
+      let (lsa, maybeElist) = fromMaybe (lsaDefault, Nothing) rest
+      let connecting = has (csPingStatus . _PingConnecting) cs
+      let elist = Just (Text.pack (fromMaybe "" maybeElist))
+      let cached = elist == view (csChannelList . clsElist) cs
+      let sendM = sendMsg cs (ircList (Text.pack <$> maybeToList maybeElist))
+      unless (connecting || (cached && not (_lsaRefresh lsa))) sendM
+      let cs' = set (csChannelList . clsElist) elist cs 
+      let subfocus = FocusChanList (view csNetwork cs) (_lsaMin lsa) (_lsaMax lsa)
+      commandSuccessUpdateCS cs' (changeSubfocus subfocus st)
+
+data ListArgs = ListArgs
+  { _lsaRefresh :: Bool
+  , _lsaMin     :: Maybe Int
+  , _lsaMax     :: Maybe Int
+  }
+
+lsaDefault :: ListArgs
+lsaDefault = ListArgs
+  { _lsaRefresh = False
+  , _lsaMin = Nothing
+  , _lsaMax = Nothing
+  }
+
+lsaParse :: String -> Maybe ListArgs
+lsaParse = lsaParse' lsaDefault
+  where
+    lsaParse' lsa str = case str of
+      '~':rest -> lsaParse' lsa{ _lsaRefresh = True } rest
+      ',':rest -> lsaParse' lsa rest
+      '>':(reads -> [(min', rest)]) | min' >= 0 ->
+        lsaParse' lsa{ _lsaMin = Just min'} rest
+      '<':(reads -> [(max', rest)]) | max' >= 0 ->
+        lsaParse' lsa{ _lsaMax = Just max'} rest
+      "" -> Just lsa
+      _ -> Nothing
 
 cmdLusers :: NetworkCommand (Maybe String)
 cmdLusers cs st arg =
@@ -183,10 +233,15 @@ cmdWhois cs st rest =
   do sendMsg cs (ircWhois (Text.pack <$> words rest))
      commandSuccess st
 
-cmdWho :: NetworkCommand String
-cmdWho cs st rest =
-  do sendMsg cs (ircWho (Text.pack <$> words rest))
-     commandSuccess st
+cmdWho :: NetworkCommand (Maybe (String, Maybe String))
+cmdWho cs  st Nothing = commandSuccess $ changeSubfocus (FocusWho (view csNetwork cs)) st
+cmdWho cs st (Just (query, arg)) =
+  do
+    let query' = Text.pack query
+    let arg' = fromMaybe "" arg
+    let cs' = set csWhoReply (newWhoReply query' arg') cs
+    sendMsg cs (ircWho (query' : maybeToList (Text.pack <$> arg)))
+    commandSuccessUpdateCS cs' $ changeSubfocus (FocusWho (view csNetwork cs)) st
 
 cmdWhowas :: NetworkCommand String
 cmdWhowas cs st rest =
