@@ -36,7 +36,9 @@ module Client.State
   , clientIgnores
   , clientIgnoreMask
   , clientConnection
+  , clientNotifications
   , clientBell
+  , clientUiFocused
   , clientExtensions
   , clientRegex
   , clientLogQueue
@@ -54,6 +56,8 @@ module Client.State
   , withClientState
   , clientIsFiltered
   , clientFilter
+  , clientFilterChannels
+  , clientNetworkPalette
   , buildMatcher
   , clientToggleHideMeta
   , channelUserList
@@ -74,6 +78,8 @@ module Client.State
   , clientAutoconnects
   , clientActiveCommand
   , clientNextWindowName
+  , clientWindowHint
+  , clientHelp
 
   , clientExtraFocuses
   , currentNickCompletionMode
@@ -104,11 +110,6 @@ module Client.State
   , esActive
   , esMVar
   , esStablePtr
-
-  -- * URL view
-  , urlPattern
-  , urlMatches
-
   ) where
 
 import           Client.CApi
@@ -117,6 +118,7 @@ import           Client.Configuration
 import           Client.Configuration.ServerSettings
 import           Client.Configuration.Sts
 import           Client.Image.Message
+import           Client.Image.PackedImage (imageText)
 import           Client.Image.Palette
 import           Client.Log
 import           Client.Mask
@@ -125,15 +127,17 @@ import           Client.Network.Async
 import           Client.State.Channel
 import qualified Client.State.EditBox as Edit
 import           Client.State.Focus
+import           Client.State.Help
 import           Client.State.Network
 import           Client.State.Window
+import           Client.State.Target (MessageTarget(..))
 import           ContextFilter
 import           Control.Applicative
 import           Control.Concurrent.MVar
 import           Control.Concurrent.STM
 import           Control.Exception
 import           Control.Lens
-import           Control.Monad
+import           Control.Monad()
 import           Data.Foldable
 import           Data.HashMap.Strict (HashMap)
 import qualified Data.HashMap.Strict as HashMap
@@ -152,7 +156,7 @@ import           Data.Time
 import           Foreign.StablePtr
 import           Irc.Codes
 import           Irc.Identifier
-import           Irc.Message
+import           Irc.Message hiding (MessageTarget(..))
 import           Irc.RawIrcMsg
 import           Irc.UserInfo
 import           LensUtils
@@ -161,7 +165,6 @@ import qualified System.Random as Random
 import           Text.Regex.TDFA
 import           Text.Regex.TDFA.String (compile)
 
-
 -- | All state information for the IRC client
 data ClientState = ClientState
   { _clientWindows           :: !(Map Focus Window) -- ^ client message buffers
@@ -192,7 +195,9 @@ data ClientState = ClientState
   , _clientEditMode          :: EditMode                  -- ^ editor rendering mode
   , _clientEditLock          :: Bool                      -- ^ editor locked and won't send
 
-  , _clientBell              :: !Bool                     -- ^ sound a bell next draw
+  , _clientNotifications     :: [(LText.Text, LText.Text)] -- ^ notifications to send next draw
+  , _clientBell              :: !Bool                     -- ^ terminal bell on next redraw
+  , _clientUiFocused         :: !Bool                     -- ^ whether the UI is focused; used by notifications
 
   , _clientIgnores           :: !(HashSet Identifier)     -- ^ ignored masks
   , _clientIgnoreMask        :: Mask                      -- ^ precomputed ignore regular expression (lazy)
@@ -204,6 +209,7 @@ data ClientState = ClientState
 
   , _clientStsPolicy         :: !(HashMap Text StsPolicy) -- ^ STS policy entries
   , _clientHighlights        :: !(HashMap Identifier Highlight) -- ^ highlights
+  , _clientHelp              :: !HelpState                -- ^ cached help text
   }
 
 data Matcher = Matcher
@@ -282,13 +288,16 @@ withClientState cfgPath cfg k =
         , _clientEditLock          = False
         , _clientActivityBar       = view configActivityBar cfg
         , _clientShowPing          = view configShowPing cfg
+        , _clientNotifications     = []
         , _clientBell              = False
+        , _clientUiFocused         = True
         , _clientExtensions        = exts
         , _clientLogQueue          = []
         , _clientErrorMsg          = Nothing
         , _clientRtsStats          = Nothing
         , _clientStsPolicy         = sts
         , _clientHighlights        = HashMap.empty
+        , _clientHelp              = makeHelp Nothing []
         }
 
 withExtensionState :: (ExtensionState -> IO a) -> IO a
@@ -322,6 +331,7 @@ recordSuccess now ste m =
     , _msgNetwork = ""
     } ste
 
+
 -- | Add a message to the window associated with a given channel
 recordChannelMessage ::
   Text       {- ^ network -} ->
@@ -329,9 +339,18 @@ recordChannelMessage ::
   ClientMessage ->
   ClientState ->
   ClientState
-recordChannelMessage network channel msg st
+recordChannelMessage = recordChannelMessage' True
+
+recordChannelMessage' ::
+  Bool       {- ^ create  -} ->
+  Text       {- ^ network -} ->
+  Identifier {- ^ channel -} ->
+  ClientMessage ->
+  ClientState ->
+  ClientState
+recordChannelMessage' create network channel msg st
   = recordLogLine msg statusModes channel'
-  $ recordWindowLine focus wl st
+  $ recordWindowLine' create focus wl st
   where
     focus      = ChannelFocus network channel'
     wl         = toWindowLine rendParams importance msg
@@ -342,6 +361,8 @@ recordChannelMessage network channel msg st
       , rendHighlights  = highlights
       , rendPalette     = clientPalette st
       , rendAccounts    = accounts
+      , rendNetPalette  = clientNetworkPalette st
+      , rendChanTypes   = "#&!+" -- TODO: Don't hardcode this, use CHANTYPES ISUPPORT.
       }
 
     -- on failure returns mempty/""
@@ -418,8 +439,18 @@ msgImportance msg st =
                      | otherwise -> WLBoring
         Kick _ _ kicked _ | isMe kicked -> WLImportant
                           | otherwise   -> WLNormal
+        Invite _ tgt _    | isMe tgt    -> WLImportant
+                          | otherwise   -> WLNormal
         Error{} -> WLImportant
 
+        -- away notices
+        Reply _ RPL_AWAY _     -> WLBoring
+
+        -- list output
+        Reply _ RPL_LISTSTART _ -> WLBoring
+        Reply _ RPL_LIST      _ -> WLBoring
+        Reply _ RPL_LISTEND   _ -> WLBoring
+
         -- channel information
         Reply _ RPL_TOPIC _    -> WLBoring
         Reply _ RPL_INVITING _ -> WLBoring
@@ -443,6 +474,7 @@ ircIgnorable msg !st =
     Ctcp who _ "ACTION" _ -> checkUser who
     -- notice ctcp responses are not already metadata
     CtcpNotice who _ _ _ -> checkUser who
+    Invite who _ _       -> checkUser who
     _                    -> Nothing
   where
     checkUser !who
@@ -468,20 +500,11 @@ recordIrcMessage ::
 recordIrcMessage network target msg st =
   updateTransientError (NetworkFocus network) msg $
   case target of
-    TargetNetwork     -> recordNetworkMessage msg st
-    TargetWindow chan -> recordChannelMessage network chan msg st
-    TargetUser user   ->
-      foldl' (\st' chan -> overStrict
-                             (clientWindows . ix (ChannelFocus network chan))
-                             (addToWindow wl) st')
-           st chans
-      where
-        wl    = toWindowLine' network st WLBoring msg
-        chans = user
-              : case preview (clientConnection network . csChannels) st of
-                  Nothing -> []
-                  Just m  -> [chan | (chan, cs) <- HashMap.toList m
-                                   , HashMap.member user (view chanUsers cs) ]
+    TargetDrop         -> st
+    TargetNetwork      -> recordNetworkMessage msg st
+    TargetExisting win -> recordChannelMessage' False network win  msg st
+    TargetWindow chan  -> recordChannelMessage' True  network chan msg st
+    TargetUser user    -> recordUserMessage network user msg st
 
 -- | Compute the sigils of the user who sent a message.
 computeMsgLineSigils ::
@@ -540,6 +563,27 @@ recordNetworkMessage msg st = updateTransientError focus msg
     importance = msgImportance msg st
     wl         = toWindowLine' network st importance msg
 
+-- | Record a message on every window where a user is present.
+recordUserMessage ::
+  Text       {- ^ network -} ->
+  Identifier {- ^ user -} ->
+  ClientMessage ->
+  ClientState ->
+  ClientState
+recordUserMessage network user msg st = foldl' foldFn st chans
+  where
+    -- FIXME: We discard the the boolean from addToWindow here,
+    -- which means notifications for important cross-channel activity never happen.
+    -- This currently affects nothing AFAIK, but who knows what the future holds?
+    windowsLens chan = clientWindows . ix (ChannelFocus network chan)
+    foldFn st' chan = overStrict (windowsLens chan) (fst . addToWindow wl) st'
+    wl    = toWindowLine' network st WLBoring msg
+    chans = user
+          : case preview (clientConnection network . csChannels) st of
+              Nothing -> []
+              Just m  -> [chan | (chan, cs) <- HashMap.toList m
+                               , HashMap.member user (view chanUsers cs) ]
+
 recordError ::
   ZonedTime       {- ^ now             -} ->
   Text            {- ^ network         -} ->
@@ -553,40 +597,79 @@ recordError now net msg =
     , _msgBody    = ErrorBody msg
     }
 
-clientNextWindowName :: ClientState -> Char
-clientNextWindowName st =
-  case clientWindowNames st \\ usedNames of
-    []  -> '\0'
-    c:_ -> c
+clientNextWindowName :: Maybe WindowHint -> ClientState -> Char
+clientNextWindowName hint st
+  | Just n <- windowHintName =<< hint, n `notElem` usedNames = n
+  | c:_ <- availableNames \\ usedNames    = c
+  | otherwise                             = '\0'
   where
     usedNames = toListOf (clientWindows . folded . winName . _Just) st
 
+    availableNames :: String
+    availableNames = clientWindowNames st \\ reservedNames
+
+    reservedNames :: String
+    reservedNames =
+      toListOf (clientConnections . folded . csSettings . ssWindowHints . folded . to windowHintName  . folded) st
+
+clientWindowHint :: Focus -> ClientState -> Maybe WindowHint
+clientWindowHint focus st =
+ do net <- focusNetwork focus
+    let hintFocus =
+          case focus of
+            Unfocused -> Unfocused
+            NetworkFocus {} -> NetworkFocus ""
+            ChannelFocus _ x -> ChannelFocus "" x
+    preview (clientConnection net . csSettings . ssWindowHints . ix hintFocus) st
+
 -- | Record window line at the given focus creating the window if necessary
 recordWindowLine ::
   Focus ->
   WindowLine ->
   ClientState ->
   ClientState
-recordWindowLine focus wl st = st2
+recordWindowLine = recordWindowLine' True
+
+recordWindowLine' ::
+  Bool ->
+  Focus ->
+  WindowLine ->
+  ClientState ->
+  ClientState
+recordWindowLine' create focus wl st = st1
   where
+    hints = clientWindowHint focus st
+    winActivity = fromMaybe AFLoud (windowHintActivity =<< hints)
+
     freshWindow = emptyWindow
-      { _winName' = clientNextWindowName st
-      , _winHideMeta = view (clientConfig . configHideMeta) st
+      { _winName'    = clientNextWindowName hints st
+      , _winHideMeta = fromMaybe (view (clientConfig . configHideMeta) st) (windowHintHideMeta =<< hints)
+      , _winHidden   = fromMaybe False (windowHintHidden =<< hints)
+      , _winActivityFilter   = winActivity
       }
 
-    st1 = over (clientWindows . at focus)
-               (\w -> Just $! addToWindow wl (fromMaybe freshWindow w))
-               st
+    add True  w = Just $! addToWindow wl (fromMaybe freshWindow w)
+    add False w = addToWindow wl <$> w
 
-    st2
-      | not (view clientBell st)
-      , view (clientConfig . configBellOnMention) st
-      , view wlImportance wl == WLImportant
-      , not (hasMention st) = set clientBell True st1
+    addedMaybe = add create $ view (clientWindows . at focus) st
+    st1 = case addedMaybe of
+      Just (w', notify) -> addNotify notify focus wl $ set (clientWindows . at focus) (Just w') st
+      Nothing -> st
 
-      | otherwise = st1
-
-    hasMention = elemOf (clientWindows . folded . winMention) WLImportant
+addNotify :: Bool -> Focus -> WindowLine -> ClientState -> ClientState
+addNotify False _     _  st = st
+addNotify True  focus wl st
+  | focus == view clientFocus st && view clientUiFocused st = st
+  | otherwise = addBell $ over clientNotifications (cons (focusText focus, bodyText)) st
+  where
+    addBell st'
+      | not (view clientBell st')
+      , view (clientConfig . configBellOnMention) st' = set clientBell True st'
+      | otherwise = st'
+    bodyText = imageText (view wlPrefix wl) <> " " <> imageText (view wlImage wl)
+    focusText Unfocused = "Application Notice"
+    focusText (NetworkFocus net) = LText.fromChunks ["Notice from ", net]
+    focusText (ChannelFocus net chan) = LText.fromChunks ["Activity on ", net, ":", idText chan]
 
 toWindowLine :: MessageRendererParams -> WindowLineImportance -> ClientMessage -> WindowLine
 toWindowLine params importance msg = WindowLine
@@ -611,9 +694,11 @@ toWindowLine' network st =
 
 -- | Function applied to the client state every redraw.
 clientTick :: ClientState -> ClientState
-clientTick = set clientBell False
-           . markSeen
+clientTick st = (if view clientUiFocused st then markSeen else id)
+           . set clientBell False
+           . set clientNotifications []
            . set clientLogQueue []
+           $ st
 
 
 -- | Mark the messages on the current window (and any splits) as seen.
@@ -670,7 +755,9 @@ clientMatcher ::
   Maybe Matcher {- ^ optional predicate -}
 clientMatcher st =
   case clientActiveCommand st of
+    -- Refer to the grep command in Client.Commands.Window
     Just ("grep" , reStr) -> buildMatcher reStr
+    Just ("g" , reStr)    -> buildMatcher reStr
     _ -> case view clientRegex st of
            Nothing -> Nothing
            Just r  -> Just r
@@ -692,6 +779,19 @@ clientFilter st f xs =
      where
        limit = maybe id take (matcherMax m)
 
+clientFilterChannels ::
+  ClientState ->
+  Maybe Int ->
+  Maybe Int ->
+  [(Identifier, Int, Text)] ->
+  [(Identifier, Int, Text)]
+clientFilterChannels st min' (Just max') =
+  filter (\(_, users, _) -> users < max') . clientFilterChannels st min' Nothing
+clientFilterChannels st (Just min') Nothing =
+  filter (\(_, users, _) -> users > min') . clientFilterChannels st Nothing Nothing
+clientFilterChannels st Nothing Nothing = clientFilter st filterOn
+  where filterOn (chan, _, topic) = LText.fromChunks [idText chan, " ", topic]
+
 data MatcherArgs = MatcherArgs
   { argAfter     :: !Int
   , argBefore    :: !Int
@@ -752,33 +852,6 @@ clientActiveCommand st =
     _                -> Nothing
 
 
--- | Regular expression for matching HTTP/HTTPS URLs in chat text.
-urlPattern :: Regex
-Right urlPattern =
-  compile
-    defaultCompOpt
-    defaultExecOpt{captureGroups=False}
-    "https?://([[:alnum:]-]+\\.)*([[:alnum:]-]+)(:[[:digit:]]+)?(/[-0-9a-zA-Z$_.+!*'(),%?&=:@/;~#]*)?|\
-    \<https?://[^>]*>|\
-    \\\(https?://[^\\)]*\\)"
-
-
--- | Find all the URL matches using 'urlPattern' in a given 'Text' suitable
--- for being opened. Surrounding @<@ and @>@ are removed.
-urlMatches :: LText.Text -> [Text]
-urlMatches txt = removeBrackets . extractText . (^?! ix 0)
-             <$> matchAll urlPattern (LText.unpack txt)
-  where
-    extractText (off,len) = LText.toStrict
-                          $ LText.take (fromIntegral len)
-                          $ LText.drop (fromIntegral off) txt
-
-    removeBrackets t =
-      case Text.uncons t of
-       Just ('<',t') | not (Text.null t') -> Text.init t'
-       Just ('(',t') | not (Text.null t') -> Text.init t'
-       _                                  -> t
-
 -- | Remove a network connection and unlink it from the network map.
 -- This operation assumes that the network connection exists and should
 -- only be applied once per connection.
@@ -851,12 +924,13 @@ applyMessageToClientState ::
   Text                       {- ^ network name             -} ->
   NetworkState               {- ^ network connection state -} ->
   ClientState                {- ^ client state             -} ->
-  ([RawIrcMsg], ClientState) {- ^ response , DCC updates, updated state -}
+  ([RawIrcMsg], ClientState) {- ^ response , updated state -}
 applyMessageToClientState time irc network cs st =
   cs' `seq` (reply, st')
   where
     Apply reply cs' = applyMessage time irc cs
     st' = applyWindowRenames network irc
+        . applyHelpIfAwaiting network irc
         $ set (clientConnections . ix network) cs' st
 
 -- | When a nick change happens and there is an open query window for that nick
@@ -888,6 +962,10 @@ applyWindowRenames network (Nick old new) st
 
 applyWindowRenames _ _ st = st
 
+applyHelpIfAwaiting :: Text {- ^ network -} -> IrcMsg -> ClientState -> ClientState
+applyHelpIfAwaiting network irc st
+  | awaitingHelp (_clientHelp st) == Just network = over clientHelp (applyHelpReply (clientPalette st) irc) st
+  | otherwise = st
 
 ------------------------------------------------------------------------
 -- Scrolling
@@ -914,17 +992,21 @@ clientExtraFocuses st =
 -- considered important and will be jumped to first.
 jumpToActivity :: ClientState -> ClientState
 jumpToActivity st =
-  case mplus highPriority lowPriority of
-    Just (focus,_) -> changeFocus focus st
+  case locate (Nothing, 1) windowList of
+    Just focus -> changeFocus focus st
     Nothing ->
       case view clientActivityReturn st of
         Just focus -> changeFocus focus st
         Nothing    -> st
   where
-    windowList   = filter (not . view winSilent . snd)
-                 $ views clientWindows Map.toAscList st
-    highPriority = find (\x -> WLImportant == view winMention (snd x)) windowList
-    lowPriority  = find (\x -> view winUnread (snd x) > 0) windowList
+    windowList = views clientWindows Map.toAscList st
+    locate (v, _) [] = v
+    locate vp@(_, vRank) ((f,w):wins)
+      | fRank == 5 = Just f -- Short circuit
+      | fRank > vRank = locate (Just f, fRank) wins
+      | otherwise = locate vp wins
+      where
+        fRank = fromEnum (isJust $ view winName w) + 2 * fromEnum (view winMention w)
 
 -- | Jump the focus directly to a window based on its zero-based index
 -- while ignoring hidden windows.
@@ -1103,3 +1185,9 @@ clientAutoconnects st =
 clientToggleHideMeta :: ClientState -> ClientState
 clientToggleHideMeta st =
   overStrict (clientWindows . ix (view clientFocus st) . winHideMeta) not st
+
+-- | Generates the NetworkPalette for the current focus.
+clientNetworkPalette :: ClientState -> NetworkPalette
+clientNetworkPalette st = case focusNetwork (view clientFocus st) of
+  Just net -> configNetworkPalette net (view clientConfig st)
+  Nothing  -> defaultNetworkPalette
