@@ -17,14 +17,15 @@ module Client.State.Channel
   (
   -- * Channel state
     ChannelState(..)
+  , chanCreation
+  , chanJoined
+  , chanLists
+  , chanModes
+  , chanQueuedModeration
   , chanTopic
   , chanTopicProvenance
   , chanUrl
   , chanUsers
-  , chanModes
-  , chanLists
-  , chanCreation
-  , chanQueuedModeration
 
   -- * Mask list entries
   , MaskListEntry(..)
@@ -44,21 +45,23 @@ module Client.State.Channel
   , nickChange
   ) where
 
-import           Control.Lens
-import           Data.HashMap.Strict
-import qualified Data.HashMap.Strict as HashMap
-import qualified Data.Map.Strict as Map
-import           Data.Map.Strict (Map)
-import           Data.Text (Text)
-import qualified Data.Text as Text
-import           Data.Time
-import           Irc.Identifier
-import           Irc.RawIrcMsg (RawIrcMsg)
-import           Irc.UserInfo
+import Control.Lens ((&), sans, (<<.~), over, set, makeLenses, At(at))
+import Data.HashMap.Strict (HashMap)
+import Data.HashMap.Strict qualified as HashMap
+import Data.Map.Strict (Map)
+import Data.Map.Strict qualified as Map
+import Data.Text (Text)
+import Data.Text qualified as Text
+import Data.Time (UTCTime)
+import Irc.Identifier (Identifier)
+import Irc.RawIrcMsg (RawIrcMsg)
+import Irc.UserInfo (UserInfo)
 
 -- | Dynamic information about the state of an IRC channel
 data ChannelState = ChannelState
-  { _chanTopic :: !Text
+  { _chanJoined :: !Bool
+        -- ^ client is currently connected to this channel
+  , _chanTopic :: !Text
         -- ^ topic text
   , _chanTopicProvenance :: !(Maybe TopicProvenance)
         -- ^ author and timestamp for topic
@@ -94,7 +97,8 @@ makeLenses ''MaskListEntry
 -- | Construct an empty 'ChannelState'
 newChannel :: ChannelState
 newChannel = ChannelState
-  { _chanTopic = Text.empty
+  { _chanJoined = False
+  , _chanTopic = Text.empty
   , _chanUrl = Nothing
   , _chanTopicProvenance = Nothing
   , _chanUsers = HashMap.empty
