@@ -19,7 +19,10 @@ module Client.State.Focus
   , WindowsFilter(..)
 
   -- * Focus operations
+  , parseFocus
   , focusNetwork
+  , actualFocus
+  , isPrefixOfFocus
 
   -- * Focus Prisms
   , _ChannelFocus
@@ -27,9 +30,10 @@ module Client.State.Focus
   , _Unfocused
   ) where
 
-import Control.Lens (makePrisms)
-import Data.Text (Text)
-import Irc.Identifier (Identifier)
+import           Control.Lens (makePrisms, (<&>))
+import           Data.Text (Text)
+import qualified Data.Text as Text
+import           Irc.Identifier (Identifier, idPrefix, mkId)
 
 -- | Currently focused window
 data Focus
@@ -43,20 +47,20 @@ makePrisms ''Focus
 -- | Subfocus view
 data Subfocus
   = FocusMessages    -- ^ Show messages
-  | FocusInfo        -- ^ Show channel metadata
-  | FocusUsers       -- ^ Show channel user list
-  | FocusMasks !Char -- ^ Show channel mask list for given mode
+  | FocusInfo  !Text !Identifier       -- ^ Show channel metadata
+  | FocusUsers !Text !Identifier       -- ^ Show channel user list
+  | FocusMasks !Text !Identifier !Char -- ^ Show channel mask list for given mode
   | FocusWindows WindowsFilter -- ^ Show client windows
   | FocusPalette     -- ^ Show current palette
   | FocusMentions    -- ^ Show all mentions
   | FocusDigraphs    -- ^ Show all digraphs
   | FocusKeyMap      -- ^ Show key bindings
-  | FocusHelp (Maybe Text) -- ^ Show help window with optional command
+  | FocusHelp        -- ^ Show help window
   | FocusRtsStats    -- ^ Show GHC RTS statistics
   | FocusIgnoreList  -- ^ Show ignored masks
   | FocusCert        -- ^ Show rendered certificate
-  | FocusChanList (Maybe Int) (Maybe Int) -- ^ Show channel list
-  | FocusWho -- ^ Show last reply to a WHO query
+  | FocusChanList !Text (Maybe Int) (Maybe Int) -- ^ Show channel list
+  | FocusWho !Text -- ^ Show last reply to a WHO query
   deriving (Eq,Show)
 
 -- | Unfocused first, followed by focuses sorted by network.
@@ -79,6 +83,41 @@ focusNetwork Unfocused = Nothing
 focusNetwork (NetworkFocus network) = Just network
 focusNetwork (ChannelFocus network _) = Just network
 
+-- | Returns what focus a subfocus is actually for.
+actualFocus :: Subfocus -> Focus -> Focus
+actualFocus sf = case sf of
+  FocusInfo  net chan   -> const (ChannelFocus net chan)
+  FocusUsers net chan   -> const (ChannelFocus net chan)
+  FocusMasks net chan _ -> const (ChannelFocus net chan)
+  FocusChanList net _ _ -> const (NetworkFocus net)
+  FocusWho net          -> const (NetworkFocus net)
+  _ -> id
+
+-- | Parses a single focus name given a default network.
+parseFocus ::
+  Maybe Text {- ^ default network    -} ->
+  String {- ^ @[network:]target@ -} ->
+  Maybe Focus
+parseFocus mbNet x =
+  case break (==':') x of
+    ("*","")     -> pure Unfocused
+    (net,_:"")   -> pure (NetworkFocus (Text.pack net))
+    (net,_:chan) -> pure (ChannelFocus (Text.pack net) (mkId (Text.pack chan)))
+    (chan,"")    -> mbNet <&> \net ->
+                    ChannelFocus net (mkId (Text.pack chan))
+
+isPrefixOfFocus :: String -> Focus -> Bool
+isPrefixOfFocus prefix focus = case break (==':') prefix of
+  ("","")  -> True
+  ("*","") -> focus == Unfocused
+  (chan,"") -> case focus of 
+    ChannelFocus _    chanF -> idPrefix (mkId $ Text.pack chan) chanF
+    NetworkFocus netF       -> Text.isPrefixOf (Text.pack chan) netF
+    Unfocused               -> False
+  (net,_:chan) -> case focus of
+    ChannelFocus netF chanF -> netF == Text.pack net && idPrefix (mkId $ Text.pack chan) chanF
+    _ -> False
+
 -- | Filter argument for 'FocusWindows'
 data WindowsFilter
   = AllWindows     -- ^ no filter
