@@ -14,12 +14,13 @@ module Client.View.Cert
 
 import           Client.Image.PackedImage
 import           Client.Image.Palette
+import           Client.Image.MircFormatting
 import           Client.State
 import           Client.State.Focus
 import           Client.State.Network
 import           Control.Lens
 import           Data.Text (Text)
-import           Graphics.Vty.Attributes
+import qualified Data.Text.Lazy as LText
 
 -- | Render the lines used in a channel mask list
 certViewLines ::
@@ -29,7 +30,8 @@ certViewLines st
   , Just cs <- preview (clientConnection network) st
   , let xs = view csCertificate cs
   , not (null xs)
-  = text' defAttr <$> xs
+  = map (parseIrcText pal)
+  $ clientFilter st LText.fromStrict xs
 
   | otherwise = [text' (view palError pal) "No certificate available"]
   where
